package com.aitrainer.api.controller

import com.aitrainer.api.model.Customer
import com.aitrainer.api.model.User
import com.aitrainer.api.service.ServiceBeans
import com.aitrainer.api.repository.CustomerRepository
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpHeaders
import org.springframework.http.ResponseEntity
import org.springframework.security.access.annotation.Secured
import org.springframework.web.bind.annotation.*
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.*
import javax.validation.Valid


@RestController
@RequestMapping("/api")
class CustomerController ( private val customerRepository: CustomerRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @Autowired
    var serviceBeans: ServiceBeans? = null

    @Secured
    @GetMapping("/customers")
    fun getAllCustomers(@RequestHeader headers: HttpHeaders): List<Customer> =
            customerRepository.findAll()

    @Secured
    @PostMapping("/customers")
    fun createNewCustomer(@Valid @RequestBody customer: Customer, @RequestHeader headers: HttpHeaders): Customer =
            customerRepository.save(customer)

    @Secured
    @GetMapping("/customers/{id}")
    fun getCustomerById(@PathVariable(value = "id") customerId: Long, @RequestHeader headers: HttpHeaders): ResponseEntity<Customer> {
        val customer: Customer? = customerRepository.findById(customerId).orElse(null)
        return if (customer == null) ResponseEntity.notFound().build() else ResponseEntity.ok().body(customer)
    }

    @Secured
    @GetMapping("/customers/find_by_firebaseuid/{uid}")
    fun getCustomerByFirebaseUid(@PathVariable(value = "uid") firebaseUid: String): ResponseEntity<Customer> {
        val customer: Customer? = customerRepository.findByFirebaseUid(firebaseUid)

        return if (customer == null) ResponseEntity.notFound().build() else ResponseEntity.ok().body(customer)
    }

    @Secured
    @GetMapping("/customers/find_by_email/{email}")
    fun getCustomerByEmail(@PathVariable(value = "email") email: String): ResponseEntity<Customer> {
        val customer: Customer? = customerRepository.findByEmail(email)
        return if (customer == null) ResponseEntity.notFound().build() else ResponseEntity.ok().body(customer)
    }


    @Secured
    @GetMapping("/customers/real")
    fun getRealCustomers(active: String, @RequestHeader headers: HttpHeaders): ResponseEntity<List<Customer>> =
            ResponseEntity.ok().body(customerRepository.findByActive(active))

    @Secured
    @GetMapping("/customers/trainees/{id}")
    fun getCustomerListByTrainerId(@PathVariable(value = "id") trainerId: Long, @RequestHeader headers: HttpHeaders):
            ResponseEntity<List<Customer>> {
        val customerList: List<Customer> = customerRepository.findByTrainerId(trainerId)

        return if (customerList.isEmpty()) ResponseEntity.notFound().build() else ResponseEntity.ok().body(customerList)
    }

    @Secured
    @PostMapping("customers/update_firebase_uid/{id}")
    fun updateCustomerFirebaseUidById(@PathVariable(value = "id") customerId: Long, @Valid @RequestBody firebaseUid: String)
            : ResponseEntity<Customer> {
        logger.info("Get customer id: $customerId uid: $firebaseUid")
        val returnCustomer: Customer = customerRepository.findById(customerId).orElse(null)
                ?: return ResponseEntity.notFound().build()

        returnCustomer.firebaseUid = firebaseUid
        return ResponseEntity.ok().body(customerRepository.save(returnCustomer))
    }

    @Secured
    @PostMapping("/customers/{id}")
    fun updateCustomerById(@PathVariable(value = "id") customerId: Long,
                           @Valid @RequestBody newCustomer: Customer,
                           @RequestHeader headers: HttpHeaders): ResponseEntity<Customer> {

        val returnCustomer: Customer = customerRepository.findById(customerId).orElse(null)
                ?: return ResponseEntity.notFound().build()

        val updatedCustomer = returnCustomer.copy()
        if (newCustomer.email != null) {
                updatedCustomer.email = newCustomer.email
            }
        if (newCustomer.name != null) {
                updatedCustomer.name = newCustomer.name
            }
        if (newCustomer.firstname != null) {
                updatedCustomer.firstname = newCustomer.firstname
            }
        if (newCustomer.goal != null) {
                updatedCustomer.goal = newCustomer.goal
            }
        if (newCustomer.dateChange != null) {
                updatedCustomer.dateChange = newCustomer.dateChange
            }
        if (newCustomer.dataPolicyAllowed != null) {
                updatedCustomer.dataPolicyAllowed = newCustomer.dataPolicyAllowed
            }
        if (newCustomer.sportId != null) {
            updatedCustomer.sportId = newCustomer.sportId
        }
        if (newCustomer.emailSubscription != null) {
            updatedCustomer.emailSubscription = newCustomer.emailSubscription
        }
        if (newCustomer.syncedDate != null) {
            updatedCustomer.syncedDate = newCustomer.syncedDate
        }
        if (newCustomer.trialDate != null) {
            updatedCustomer.trialDate = newCustomer.trialDate
        }
        if (newCustomer.firebaseRegToken != null) {
            updatedCustomer.firebaseRegToken = newCustomer.firebaseRegToken
        }
        updatedCustomer.sex = newCustomer.sex
        updatedCustomer.birthYear = newCustomer.birthYear
        updatedCustomer.fitnessLevel = newCustomer.fitnessLevel

        return ResponseEntity.ok().body(customerRepository.save(updatedCustomer))
    }


    @PostMapping("/registration")
    fun registration(@Valid @RequestBody json: String): ResponseEntity<*> {
        val customer = Customer()

        val current = LocalDateTime.now()
        val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss.SSS")
        val nowFormatted = current.format(formatter)

        val newUser: User = User().fromJson(json)
        with (customer) {
            email = newUser.username
            password = serviceBeans!!.passwordEncoder().encode(newUser.password)
            firebaseUid = newUser.firebaseUid
            dateAdd = nowFormatted
            firstname = ""
            name = ""
            email = newUser.username
        }

        val returnCustomer: Customer? = customerRepository.findByEmail(newUser.username).let {
                if ( it == null) {
                    customerRepository.save(customer)
                } else {
                    null
                }
        }

        return if ( returnCustomer != null ) {
            ResponseEntity.ok().body(returnCustomer)
        } else {
            ResponseEntity.badRequest().body("Customer exists")
        }

    }

    @PostMapping("/login")
    fun login(@Valid @RequestBody json: String): ResponseEntity<*> {
        val customer = Customer()

        val newUser: User = User().fromJson(json)
        with (customer) {
            email = newUser.username
            password = newUser.password
        }
        val returnCustomer: Customer? = customerRepository.findByEmail(newUser.username).let {
            if ( it == null) {
                null
            } else {
                if (serviceBeans!!.passwordEncoder().matches(newUser.password, it.password)) {
                    it
                } else {
                    null
                }

            }
        }
        return if ( returnCustomer != null ) {
            ResponseEntity.ok().body(returnCustomer)
        } else {
            ResponseEntity.badRequest().body("Customer does not exist or the password is wrong")
        }
    }
}